{% extends "base.html" %}

{% block extra_head %}
{# These are used on album pages - prefetch now #}
<link rel="prefetch" href="static/jquery-3.3.1.min.js" />
<link rel="prefetch" href="static/galleria.min.js" />
<link rel="prefetch" href="static/themes/classic/galleria.classic.min.js" />
<link rel="prefetch" href="static/plugins/history/galleria.history.min.js" />
{% endblock %}

{% block content %}
<div id="albums">
  <ul>
    {% for alb in album.albums %}
    <li><a href="{{ alb.url }}">
      <img src="{{ alb.thumbnail }}" class="album_thumb" alt="{{ alb.name }}" title="{{ alb.name }}" /></a>
      <span class="album_title">{{ alb.title }}</span>
    </li>
    {% endfor %}
  </ul>
</div>
{% include 'album_items.html' %}
{% endblock %}
