<!doctype html>
<html lang="{{ settings.html_language }}">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="x-ua-compatible" content="ie=edge">

    <title>{{ album.title|striptags }} - {{ settings.title }}</title>
    <meta name="description" content="">
    <meta name="author" content="{{ album.author }}">
    <meta name="viewport" content="width=device-width">
    <link rel="stylesheet" href="{{ theme.url }}/css/skeleton.css">
    <link rel="stylesheet" href="{{ theme.url }}/css/colorbox.css">
    <link rel="stylesheet" href="{{ theme.url }}/css/style.css">
    {% if user_css %}
    <link rel="stylesheet" href="{{ theme.url }}/{{ user_css }}">
    {% endif %}
    {% include 'default_head.html' %}
    {% block extra_head %}{% endblock extra_head %}
    {% include 'analytics.html' %}
  </head>
  <body>
    {% include 'decrypt.html' %}
    {% include 'gtm.html' %}
    <div class="container">
      <div class="three columns">
        <div class="sidebar">
          <h1><a href="{{ album.index_url }}">
          {% if settings.site_logo %}
          <img src="{{ theme.url }}/{{ settings.site_logo }}" alt="{{ index_title }}">
          {% else %}
          {{ index_title }}
          {% endif %}
          </a></h1>
          {% include 'links.html' %}
          {% include 'footer.html' %}
        </div>
      </div>
      <div id="main" role="main" class="nine columns offset-by-three">
        <header>
          {% include 'breadcrumb.html' %}
          {% if album.breadcrumb %}
            <hr>
          {% endif %}
        </header>

        {% block content %}{% endblock %}

        {% if album.description %}
          <div id="description" class="row">
            {{ album.description }}
          </div>
        {% endif %}
      </div>
    </div>
    {% block footer %}{% endblock %}
    {% include 'piwik.html' %}
  </body>
</html>
