{% extends "base.html" %}

{% block extra_head %}
    {# We will always want to show the first image when we're on an album page #}
    {# Start loading now for faster document complete instead of waiting for js to load it #}
    <style>
    body::after{
        position:absolute; width:0; height:0; overflow:hidden; z-index:-1;
        content:url({{ album.medias[0].url }});
    }</style>
{% endblock %}

{% block content %}
  {% include 'album_items.html' %}
{% endblock %}
