<!doctype html>
<html lang="{{ settings.html_language }}">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="x-ua-compatible" content="ie=edge">

    <title>{{ album.title|striptags }} - {{ settings.title }}</title>
    <meta name="description" content="">
    <meta name="author" content="{{ album.author }}">
    <meta name="viewport" content="width=device-width">

    <link rel="stylesheet" href="{{ theme.url }}/css/normalize.css">
    <link rel="stylesheet" href="{{ theme.url }}/themes/{{ settings.galleria_theme }}/galleria.{{ settings.galleria_theme }}.css">
    <link rel="stylesheet" href="{{ theme.url }}/css/style.css">
    {% if user_css %}
    <link rel="stylesheet" href="{{ theme.url }}/{{ user_css }}">
    {% endif %}
    {% include 'default_head.html' %}
    {% block extra_head %}{% endblock extra_head %}
    {% include 'analytics.html' %}
  </head>
  <body>
    {% include 'decrypt.html' %}
    {% include 'gtm.html' %}
    <div class="container">
      <header>
        <h1><a href="{{ album.index_url }}">{{ index_title }}</a></h1>

        {% include 'links.html' %}
        {% include 'breadcrumb.html' %}
        {% if album.breadcrumb %}
        <hr>
        {% endif %}
      </header>

      <div id="main" role="main">
        {% block content %}{% endblock content %}
      </div>
      {% include 'footer.html' %}
    </div>
    {% block late_js %}{% endblock late_js %}
    {% include 'piwik.html' %}
  </body>
</html>
