<footer>
  <p>
    {% if album.author %}
      <span>&copy; {{ album.author }}</span>
    {% endif %}
    <span>Generated by <a href="{{ sigal_link }}">sigal</a></span>
    {% if 'sigal.plugins.feeds' in settings.plugins %}
      {% if settings.rss_feed %}
        <span><a href={{ settings.rss_feed.feed_url }}>RSS Feed</a></span>
      {% endif %}
      {% if settings.atom_feed %}
        <span><a href={{ settings.atom_feed.feed_url }}>Atom Feed</a></span>
      {% endif %}
    {% endif %}
  </p>
</footer>
